// MIKE,
//
// The CHECKSUM function calculates the CRC for incoming bytes stored in the
// array BYTE.  The returned CRC value can then be compared with the value
// given by the device.
//
// The other parameter required is the size (length) of the array BYTE.  This
// can be calculated using the "sizeof" function when CHECKSUM is called.
// eg. CHECKSUM(BYTE, sizeof(BYTE))
//
// The read_temp_sensor function (listed below CHECKSUM) gives an example of
// how CHECKSUM can be used to check the integrity of a reading from a DS1820
//
// - David Haywood 30AUG99


unsigned char CHECKSUM (unsigned char *BYTE, int size_of_BYTE) // Calculates DOW CRC
{                                                            // value for a stream of bytes
 // DECLARE VARIABLES
 int BYTE_loop;
 unsigned char CRC;

 // DECLARE AND SET VARIABLES
 unsigned char BYTE_ARRAY [256] =
 {
  0, 94, 188, 226, 97, 63, 221, 131, 194, 156, 126, 32, 163, 253, 31, 65,
  157, 195, 33, 127, 252, 162, 64, 30, 95, 1, 227, 189, 62, 96, 130, 220,
  35, 125, 159, 193, 66, 28, 254, 160, 225, 191, 93, 3, 128, 222, 60, 98,
  190, 224, 2, 92, 223, 129, 99, 61, 124, 34, 192, 158, 29, 67, 161, 255,
  70, 24, 250, 164, 39, 121, 155, 197, 132, 218, 56, 102, 229, 187, 89, 7,
  219, 133, 103, 57, 186, 228, 6, 88, 25, 71, 165, 251, 120, 38, 196, 154,
  101, 59, 217, 135, 4, 90, 184, 230, 167, 249, 27, 69, 198, 152, 122, 36,
  248, 166, 68, 26, 153, 199, 37, 123, 58, 100, 134, 216, 91, 5, 231, 185,
  140, 210, 48, 110, 237, 179, 81, 15, 78, 16, 242, 172, 47, 113, 147, 205,
  17, 79, 173, 243, 112, 46, 204, 146, 211, 141, 111, 49, 178, 236, 14, 80,
  175, 241, 19, 77, 206, 144, 114, 44, 109, 51, 209, 143, 12, 82, 176, 238,
  50, 108, 142, 208, 83, 13, 239, 177, 240, 174, 76, 18, 145, 207, 45, 115,
  202, 148, 118, 40, 171, 245, 23, 73, 8, 86, 180, 234, 105, 55, 213, 139,
  87, 9, 235, 181, 54, 104, 138, 212, 149, 203, 41, 119, 244, 170, 72, 22,
  233, 183, 85, 11, 136, 214, 52, 106, 43, 117, 151, 201, 74, 20, 246, 168,
  116, 42, 200, 150, 21, 75, 169, 247, 182, 232, 10, 84, 215, 137, 107, 53
 };
 // lookup array for CHECKSUM calcs

 // FUNCTION BODY;
 CRC = 0;
 for (BYTE_loop = 0; BYTE_loop < size_of_BYTE; BYTE_loop++)
 {
  CRC = BYTE_ARRAY[CRC ^ BYTE[BYTE_loop]]; // look up array for appropriate calc
 }
 return (CRC);
}


 float read_temp_sensor (unsigned char *serialnumber)
 {
 // DECLARE VARIABLES
 int serialnumber_loop;
 unsigned char BYTE [8], CRC;
 signed char degree;
 float degree_fraction, sensor_temp_reading;

 // FUNCTION BODY
 TouchReset();     // reset the bus
 TouchByte(0x55);  // "Match ROM" command = prepare to send a command to a particular temp sensor
 for (serialnumber_loop = 7; serialnumber_loop >= 0; serialnumber_loop--)
 {
  TouchByte(serialnumber [serialnumber_loop]); // send serial number for the temp sensor
 }
 TouchByte(0xBE);  // "Read Scratchpad" command = read the 9 data bytes from the scratchpad
 BYTE[0] = TouchByte(0xFF); // read the Low data byte
 BYTE[1] = TouchByte(0xFF); // read the High data byte
 BYTE[2] = TouchByte(0xFF); // read the Alarm High Byte
 BYTE[3] = TouchByte(0xFF); // read the Alarm Low Byte
 BYTE[4] = TouchByte(0xFF); // read the Reserved Byte 1
 BYTE[5] = TouchByte(0xFF); // read the Reserved Byte 2
 BYTE[6] = TouchByte(0xFF); // read the Count Remain Byte
 BYTE[7] = TouchByte(0xFF); // read the Count Per degree C
 CRC     = TouchByte(0xFF); // read the CRC
 TouchReset();     // reset the bus
 degree = BYTE[0] >> 1; // push 1/2 degree bit out of byte
 if(BYTE[1] != 0 ) // add the sign to the byte if there is one
 {
  degree = degree|0x80;
 }
 if(BYTE[7] != 0) // calculate the fractional part of the degree
 {
  degree_fraction = ((float)(BYTE[7] - BYTE[6])) / ((float)BYTE[7]);
 }
 else
 {
  degree_fraction = 0.25;
 }
 sensor_temp_reading = ((float)degree) - 0.25 + degree_fraction; // calculate the total sensor reading
 if (CRC != CHECKSUM(BYTE, sizeof(BYTE))) // check data integrity and resample if errors found
 {
  global_temp_sample(); // Send out a temp sample command to all the sensors
  delay(600);           // Wait 1 second for the temp to be sampled
  // NOTE: Maybe program should beep during wait for resample
  sensor_temp_reading = read_temp_sensor(serialnumber); // return error temperature
 }
 return (sensor_temp_reading);
}
